from typing import Literal

import numpy.typing as npt

from laddu.extensions import NLL, LikelihoodTerm
from laddu.utils.variables import CosTheta, Mandelstam, Mass, Phi, PolAngle, PolMagnitude

def BinnedGuideTerm(
    nll: NLL,
    variable: Mass | CosTheta | Phi | PolAngle | PolMagnitude | Mandelstam,
    amplitude_sets: list[list[str]],
    bins: int,
    range: tuple[float, float],
    count_sets: list[list[float]] | list[npt.NDArray],
    error_sets: list[list[float]] | list[npt.NDArray] | None,
) -> LikelihoodTerm: ...
def Regularizer(
    parameters: list[str],
    lda: float,
    p: Literal[1, 2],
    weights: npt.NDArray | None = None,
) -> LikelihoodTerm: ...

__all__ = ['BinnedGuideTerm', 'Regularizer']



from typing import Literal, overload

from laddu.amplitudes import Amplitude
from laddu.utils.variables import Angles

@overload
def Ylm(name: str, l: Literal[0], m: Literal[0], angles: Angles) -> Amplitude: ...
@overload
def Ylm(name: str, l: Literal[1], m: Literal[-1, 0, 1], angles: Angles) -> Amplitude: ...
@overload
def Ylm(
    name: str, l: Literal[2], m: Literal[-2, -1, 0, 1, 2], angles: Angles
) -> Amplitude: ...
@overload
def Ylm(
    name: str,
    l: Literal[3],
    m: Literal[-3, -2, -1, 0, 1, 2, 3],
    angles: Angles,
) -> Amplitude: ...
@overload
def Ylm(
    name: str,
    l: Literal[4],
    m: Literal[-4, -3, -2, -1, 0, 1, 2, 3, 4],
    angles: Angles,
) -> Amplitude: ...
@overload
def Ylm(name: str, l: int, m: int, angles: Angles) -> Amplitude: ...



from laddu.laddu import (
    KopfKMatrixA0,
    KopfKMatrixA2,
    KopfKMatrixF0,
    KopfKMatrixF2,
    KopfKMatrixPi1,
    KopfKMatrixRho,
)

__all__ = [
    'KopfKMatrixA0',
    'KopfKMatrixA2',
    'KopfKMatrixF0',
    'KopfKMatrixF2',
    'KopfKMatrixPi1',
    'KopfKMatrixRho',
]



from laddu.amplitudes import Amplitude
from laddu.utils.variables import Mandelstam, Mass

def PhaseSpaceFactor(
    name: str,
    recoil_mass: Mass,
    daughter_1_mass: Mass,
    daughter_2_mass: Mass,
    resonance_mass: Mass,
    mandelstam_s: Mandelstam,
) -> Amplitude: ...
